@@ -1,7 +1,7 @@
 /**
  ** Simple entropy harvester based upon the havege RNG
  **
- ** Copyright 2018-2020 Jirka Hladky hladky DOT jiri AT gmail DOT com
+ ** Copyright 2018-2021 Jirka Hladky hladky DOT jiri AT gmail DOT com
  ** Copyright 2009-2014 Gary Wuertz gary@issiweb.com
  ** Copyright 2011-2012 BenEleventh Consulting manolson@beneleventh.com
  **
@@ -180,6 +180,8 @@ H_PTR havege_create(             /* RETURN: app state    */
    return h;
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-parameter"
 void havege_reparent(
   H_PTR hptr)
 {
@@ -191,6 +193,7 @@ void havege_reparent(
    t->main = getpid();
 #endif
 }
+#pragma GCC diagnostic pop
 
 /**
  * Destructor. In a multi-collector build, this method should be called from a signal handler
@@ -253,7 +256,7 @@ int havege_rng(            /* RETURN: number words read     */
       buffer[i] = havege_ndread((H_COLLECT *)h->collector);
    h->error = ((H_COLLECT *)h->collector)->havege_err;
 #endif
-   return h->error==(H_UINT)H_NOERR? sz : -1;
+   return h->error==(H_UINT)H_NOERR? (int) sz : -1;
 }
 /**
  * Start the entropy collector.
@@ -321,7 +324,7 @@ int  havege_status_dump(   /* RETURN: output length   */
       havege_status(hptr, &status);
       switch(topic) {
          case H_SD_TOPIC_BUILD:
-            n += snprintf(buf, len, "ver: %s; arch: %s; vend: %s; build: (%s); collect: %dK",
+            n += snprintf(buf, len, "ver: %s; arch: %s; vend: %s; build: (%s); collect: %uK",
                status.version,
                hptr->arch,
                status.vendor,
@@ -330,7 +333,7 @@ int  havege_status_dump(   /* RETURN: output length   */
                );
             break;
          case H_SD_TOPIC_TUNE:
-            n += snprintf(buf, len, "cpu: (%s); data: %dK (%s); inst: %dK (%s); idx: %d/%d; sz: %d/%d",
+            n += snprintf(buf, len, "cpu: (%s); data: %uK (%s); inst: %uK (%s); idx: %u/%u; sz: %u/%u",
                status.cpuSources,
                status.d_cache,
                status.d_cacheSources,
@@ -347,16 +350,16 @@ int  havege_status_dump(   /* RETURN: output length   */
                if (strlen(status.tot_tests)>0) {
                   n += snprintf(buf+n, len-n, "tot tests(%s): ", status.tot_tests);
                   if ((m = status.n_tests[ H_OLT_TOT_A_P] + status.n_tests[ H_OLT_TOT_A_F])>0)
-                     n += snprintf(buf+n, len-n, "A:%d/%d ", status.n_tests[ H_OLT_TOT_A_P], m);
+                     n += snprintf(buf+n, len-n, "A:%u/%u ", status.n_tests[ H_OLT_TOT_A_P], m);
                   if ((m = status.n_tests[ H_OLT_TOT_B_P] + status.n_tests[ H_OLT_TOT_B_F])>0)
-                     n += snprintf(buf+n, len, "B:%d/%d ", status.n_tests[ H_OLT_TOT_B_P], m);
+                     n += snprintf(buf+n, len, "B:%u/%u ", status.n_tests[ H_OLT_TOT_B_P], m);
                   }
                if (strlen(status.prod_tests)>0) {
                   n += snprintf(buf+n, len-n, "continuous tests(%s): ", status.prod_tests);
                   if ((m = status.n_tests[ H_OLT_PROD_A_P] + status.n_tests[ H_OLT_PROD_A_F])>0)
-                     n += snprintf(buf+n, len-n, "A:%d/%d ", status.n_tests[ H_OLT_PROD_A_P], m);
+                     n += snprintf(buf+n, len-n, "A:%u/%u ", status.n_tests[ H_OLT_PROD_A_P], m);
                   if ((m = status.n_tests[ H_OLT_PROD_B_P] + status.n_tests[ H_OLT_PROD_B_F])>0)
-                     n += snprintf(buf+n, len, "B:%d/%d ", status.n_tests[ H_OLT_PROD_B_P], m);
+                     n += snprintf(buf+n, len, "B:%u/%u ", status.n_tests[ H_OLT_PROD_B_P], m);
                   }
                if (n>0)
                   n += snprintf(buf+n, len-n, " last entropy estimate %g", status.last_test8);
@@ -374,7 +377,7 @@ int  havege_status_dump(   /* RETURN: output length   */
                      break;
                   factor /= 1024.0;
                   }
-               n = snprintf(buf, len, "fills: %d, generated: %.4g %c bytes",
+               n = snprintf(buf, len, "fills: %u, generated: %.4g %c bytes",
                   hptr->n_fills,
                   sz / factor,
                   units[i]
@@ -406,11 +409,11 @@ const char *havege_version(const char *version)
       H_UINT p, p_interface, p_revision, p_patch;
       
 #ifdef HAVEGE_LIB_VERSION
-      sscanf(HAVEGE_LIB_VERSION, "%d:%d:%d", &l_interface, &l_revision, &l_age);
+      sscanf(HAVEGE_LIB_VERSION, "%u:%u:%u", &l_interface, &l_revision, &l_age);
 #endif
       (void)l_interface;(void)l_revision;(void)l_age;(void)p_patch;  /* No check for now */
 
-      p = sscanf(version, "%d.%d.%d", &p_interface, &p_revision, &p_patch);
+      p = sscanf(version, "%u.%u.%u", &p_interface, &p_revision, &p_patch);
       if (p!=3 || p_interface != 1 || p_revision != 9)
          return NULL;
       }
@@ -420,12 +423,16 @@ const char *havege_version(const char *version)
 /**
  * Place holder if output display not provided
  */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-parameter"
 static void havege_mute(   /* RETURN: none            */
    const char *format,     /* IN: printf format       */
    ...)                    /* IN: args                */
 {
    ;
 }
+#pragma GCC diagnostic pop
+
 #if NUMBER_CORES > 1
 /**
  * Cleanup collector(s). In a multi-collector environment, need to kill
