@@ -1,7 +1,7 @@
 /**
  ** Simple entropy harvester based upon the havege RNG
  **
- ** Copyright 2018-2020 Jirka Hladky hladky DOT jiri AT gmail DOT com
+ ** Copyright 2018-2021 Jirka Hladky hladky DOT jiri AT gmail DOT com
  ** Copyright 2009-2014 Gary Wuertz gary@issiweb.com
  **
  ** This program is free software: you can redistribute it and/or modify
@@ -92,4 +92,9 @@ typedef struct {
   */
 void error_exit(const char *, ...);
 
+/**
+  * Execution notices - to stderr or syslog
+  */
+void print_msg(const char *, ...);
+
 #endif
