@@ -1,7 +1,7 @@
 /**
  ** Simple entropy harvester based upon the havege RNG
  **
- ** Copyright 2018-2020 Jirka Hladky hladky DOT jiri AT gmail DOT com
+ ** Copyright 2018-2021 Jirka Hladky hladky DOT jiri AT gmail DOT com
  ** Copyright 2012-2014 Gary Wuertz gary@issiweb.com
  ** Copyright 2012 BenEleventh Consulting manolson@beneleventh.com
  **
@@ -167,6 +167,8 @@ static int testsDiscard(   /* RETURN: non-zero to discard   */
 /**
  * Place holder for when report is not configured
  */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-parameter"
 static void testsMute(
    H_COLLECT * h_ctxt,     /* IN-OUT: collector context     */
    H_UINT action,          /* IN: A_RUN or B_RUN            */
@@ -176,6 +178,7 @@ static void testsMute(
 {
    ;
 }
+#pragma GCC diagnostic pop
 /**
  * The public wrapper that runs the tests. On the first call, the necessary machinery is built.
  * The calls to aisTest() actually run the tests. The test shared structure is read only in this
@@ -251,10 +254,12 @@ static H_UINT aisProcedureA(  /* RETURN: bits used             */
       case TEST_INIT:
          p->bytesUsed = 0;
          p->procRetry = 0;
+         /* fallthrough */
       case TEST_RETRY:
          p->procState = TEST_INPUT;
          p->testState = TEST_INIT;
          p->testId = p->testRun   = 0;
+         /* fallthrough */
       case TEST_INPUT:
          p->data  = (H_UINT8 *)buffer;
          p->range = sz * sizeof(H_UINT) <<3;
@@ -277,6 +282,7 @@ static H_UINT aisProcedureA(  /* RETURN: bits used             */
             else if (p->testState == TEST_INPUT)
                return 0;
             }
+            /* fallthrough */
       case TEST_EVAL:
          p->procState = TEST_DONE;
          for (r = i = 0;i<p->testRun;i++)
@@ -330,10 +336,12 @@ static H_UINT aisProcedureB(  /* RETURN: bits used             */
       case TEST_INIT:
          p->bitsUsed = 0;
          p->procRetry = 0;
+         /* fallthrough */
       case TEST_RETRY:
          p->testId = p->testNbr = 0;
          p->procState = TEST_INPUT;
          p->testState = TEST_INIT;
+         /* fallthrough */
       case TEST_INPUT:
          p->noise = buffer;
          p->range = sz * BITS_PER_H_UINT;
@@ -353,6 +361,7 @@ static H_UINT aisProcedureB(  /* RETURN: bits used             */
          context->szCarry = ct;
          if (p->testState == TEST_INPUT)
             return 0;
+         /* fallthrough */
       case TEST_EVAL:
          p->procState = TEST_DONE;
          for (i=r=0;i<p->testNbr;i++)
@@ -408,6 +417,7 @@ static H_UINT aisSeq(      /* RETURN: last bit index  */
          p->full = 0;
          p->testState = TEST_INPUT;
          SAVE(0,0,0);
+         /* fallthrough */
       case TEST_INPUT:
          RESTORE(j, hilf, deadman);
          offs %= p->range;
@@ -451,6 +461,7 @@ static H_UINT aisSeq(      /* RETURN: last bit index  */
          p->bitsUsed += i;
          if (p->testState == TEST_INPUT)
             break;
+         /* fallthrough */
       case TEST_EVAL:
          if (tid==1) {
             double q[2];
@@ -573,7 +584,7 @@ static H_UINT aisTest(     /* RETURN: nz if input needed    */
  * helpers are used to implement a high performance copyBit().
  */
 #define COPY_BYTE()           {c = (*src<<bit_diff_ls)|(*(src+1)>>bit_diff_rs);src++;}
-#define COPY_FIRST()          if (xfr >= (8 - dst_bits)) {\
+#define COPY_FIRST()          if ( (int) xfr >= (8 - dst_bits)) {\
                                  *dst &= rm[dst_bits];\
                                  xfr -= 8 - dst_bits;\
                                  }\
@@ -690,10 +701,12 @@ static H_UINT fips140(     /* RETURN: updated bit offset */
       case TEST_INIT:
          p->testState = TEST_INPUT;
          p->bridge = 0;
+         /* fallthrough */
       case TEST_INPUT:
          offs = copyBits(p, offs, FIPS_USED);
          if (p->testState!=TEST_EVAL)
             break;
+         /* fallthrough */
       case TEST_EVAL:
          maxRun = ones = runLength = 0;
          memset(poker, 0, 16*sizeof(H_UINT));
@@ -765,10 +778,12 @@ static H_UINT test0(       /* RETURN: updated bit offset */
       case TEST_INIT:
          p->testState = TEST_INPUT;
          p->bridge = 0;
+         /* fallthrough */
       case TEST_INPUT:
          offs = copyBits(p, offs, TEST0_USED);
          if (p->testState!=TEST_EVAL)
             break;
+         /* fallthrough */
       case TEST_EVAL:
          qsort(p->aux, TEST0_LENGTH, 6, test0cmp);
          for (i=6,j=0;i<TEST0_LENGTH && j==0;i+=6)
@@ -817,7 +832,7 @@ static H_UINT test5(       /* RETURN: updated bit offset    */
     * This test always uses the same data as test1 through test4
     */
    for (max = k = 0,tau=1;tau<=TEST5_LENGTH;tau++){
-      Z_tau = abs(test5XOR(dp, tau) - 2500);
+      Z_tau = abs( (int) test5XOR(dp, tau) - 2500);
       if (Z_tau > max) {
          max = Z_tau;
          k = tau - 1;
@@ -888,6 +903,7 @@ static H_UINT test6a(      /* RETURN: bit offset      */
       case TEST_INIT:
          j = p->counter[0] = 0;
          p->testState = TEST_INPUT;
+         /* fallthrough */
       case TEST_INPUT:
          {
             BITSTREAM_OPEN(p->noise,offs);
@@ -900,7 +916,7 @@ static H_UINT test6a(      /* RETURN: bit offset      */
                }
             /* align to a word boundary, then shift gears to gobble words */
             while((i+8) < r && (j+8) < AIS_LENGTH) {
-               if (0==((void *)bitstream_src - (void *)p->noise) % sizeof(H_UINT))
+               if (0==((char *)bitstream_src - (char *)p->noise) % sizeof(H_UINT))
                   break;
                SIDEWAYS_ADD(c, *bitstream_src++);
                p->counter[0] += c;
@@ -923,6 +939,7 @@ static H_UINT test6a(      /* RETURN: bit offset      */
                break;
                }
          }
+         /* fallthrough */
       case TEST_EVAL:
          p->results[p->testNbr].finalValue = (double)(p->counter[0]) / (double) AIS_LENGTH;
          r = tid << 8;
@@ -958,6 +975,7 @@ static H_UINT test8(       /* RETURN: bit offset      */
          memset(p->lastpos, 0, 256*sizeof(H_UINT));
          SAVE8(0,0,0,0.0);
          p->testState  = TEST_INPUT;
+         /* fallthrough */
       case TEST_INPUT:
          RESTORE8(k,j,hilf,TG);
          r = p->range - offs;
@@ -998,6 +1016,7 @@ static H_UINT test8(       /* RETURN: bit offset      */
          p->bitsUsed += i;
          if (p->testState == TEST_INPUT)
             break;
+         /* fallthrough */
       case TEST_EVAL:
          tps->lastCoron = p->results[p->testNbr].finalValue = TG/(double)K;
          r = tid<<8;
